function[] = colloc_system_ORDA(T,w0,lam0,a,b,c,Db)

% solve the scaled model for the isotopic concentration field H(R,x)
% using the mixed spectral -- finite difference numerical method described in Sect. 2.6


% (1) parameters
T0 = 273.15 + T;            % temperature
alph = 1;                   % fractionation coefficient
gam = 1;                    % grain-boundary type

L = 2*pi/3;                 % annular sector angle

% diffusivities, m^2 s^-1
Ds = 9.1e-4 * exp(-7.2e3/(T0));   % solid (Ramseier 1967)
cons = [ 1.085e-6 1870 2.942e7 9474 ];
Dv = 1./ ( 1./(cons(1)*exp(-cons(2)./T0)) + 1./(cons(3)*exp(-cons(4)./T0)) );  % liquid (Ng 2023)
%    
betv = Dv/Ds - 1;       % diffusivity constrast: vein to ice
betb = Db/Ds - 1;       % diffusivity constrast: grain boundary to ice

% porewater flow velocity: convert m/yr to m/s
w = w0 / (3600*24*365.25);

% non-dimensionalise signal wavelength
lam = lam0 / b; 
kz = 2*pi/lam;

% dimensionless parameters
xi = a/b;           % dimensionless vein radius
epsil = c/a;        % ratio of GB thickness to vein radius
epxi = epsil * xi;  % dimensionless GB thickness
Chi = w*b/Ds;       % dimensionless vein-water flow

p1 = betv*kz^2 + i*kz*Chi;
p2 = 2*alph;
p3 = 2*(alph/gam)*epsil*(betb+1);
p4 = 2*gam;

% numerical grid parameters
J = 201; N = 100;

% transformed radial axis discretisation
Rmax = 1 - log(xi);
dR = (Rmax-1)/(J-1);
R = [1:dR:Rmax];   % transformed radial position vector


% (2) compile matrix MAT for the eigenvalue problem
% MAT*vec = s^2*vec where the column vector vec 
% consists of z_n(1)...z_n(J-1) for n from 1 to N-1 and consists of y(1)...y(J)

MAT = zeros( (N-1)*(J-1)+J , (N-1)*(J-1)+J );

[DN,x] = cheb(N);       % note that x includes the end points -1 and 1
DNorig = DN;            % for checking later
DN2 = DN^2; 
DN = DN(2:N,2:N); DN2 = DN2(2:N,2:N);   % remove first & last rows & columns

ERvec = exp(R(1:J-1)-1);
ERmat = diag(ERvec,0);      % J-1 by J-1

yPART = zeros(J-1,J);               % J-1 by J
v0 = - betb*kz^2 - 2*(betb/dR^2)*ERvec.^2;  % diag elements
v1 = (betb/dR^2 - 0.5*(betb+1)/dR) * ERvec(2:J-1).^2; % left diag elements
v2 = (betb/dR^2 + 0.5*(betb+1)/dR) * ERvec.^2; % right diag elements
v2(1) = 2*(betb/dR^2) * exp(2*(R(1)-1));
yPART(:,[1:J-1]) = yPART(:,[1:J-1]) + diag(v0,0);
yPART(:,[1:J-1]) = yPART(:,[1:J-1]) + diag(v1,-1);
yPART(:,[2:J]) = yPART(:,[2:J]) + diag(v2,0);

mainPART = zeros(J-1,J-1);          % J-1 by J-1
v0 = (2/dR^2) * ERvec.^2;
v1 = -(1/dR^2) * ERvec(2:J-1).^2; % left diag elements
v2 = -(1/dR^2) * ERvec(1:J-2).^2; % right diag elements
v2(1) = -(2/dR^2) * exp(2*(R(1)-1));
mainPART = diag(v0,0) + diag(v1,-1) + diag(v2,1);

lastrow_yPART = zeros(J,J);         % J by J
v0 = (2*(betb+1)/dR^2)*ERvec.^2 + betb*kz^2;    % diag elements
v0(J) = p1 + (p2 + p3/L)/((xi^2)*dR);    % additional final diag element
v1 = (betb+1)*ERvec(2:J-1).^2 * (-1/dR^2 + 0.5/dR); % left diag elements
v1(J-1) = -(p2 + p3/L)/((xi^2)*dR); % additional final left diag element
v2 = (betb+1)*ERvec.^2 * (-1/dR^2 - 0.5/dR); % right diag elements
v2(1) = -(2*(betb+1)/dR^2) * exp(2*(R(1)-1));
lastrow_yPART = diag(v0,0) + diag(v1,-1) + diag(v2,1);

% compile main BLOCK ROWS for the z PDE
for M2 = 1:N-1
    % (a) install y-BLOCK and main diagonal BLOCK
    MAT( [(M2-1)*(J-1)+1 : M2*(J-1)] , [(M2-1)*(J-1)+1 : M2*(J-1)] ) = mainPART;
    MAT( [(M2-1)*(J-1)+1 : M2*(J-1)] , [(N-1)*(J-1)+1 : (N-1)*(J-1)+J] ) = yPART;
    % (b) install sigma-BLOCKS (triangle symbol in the notes) and z_xx-BLOCKS (square symbol)
    for M1 = 1:N-1 
        % go across the block columns M1, on block row M2
        MAT( [(M2-1)*(J-1)+1 : M2*(J-1)] , [(M1-1)*(J-1)+1 : M1*(J-1)] ) = ...
            MAT( [(M2-1)*(J-1)+1 : M2*(J-1)] , [(M1-1)*(J-1)+1 : M1*(J-1)] ) ...
            + (2*p4/(epxi*L)) * ((-2*(-1)^(N+M1))/(1+x(M1+1))) * ERmat ...
            - ( (2/L)^2 * DN2(M2,M1) ) * ERmat.^2;
        % check part of above expression against final row of the original Chebyshev DN matrix
        % test(M2,M1) = abs( DNorig(N+1,M1+1) - (-2*(-1)^(N+M1))/(1+x(M1+1)) );
    end
end
% max(max(test))

% compile FINAL block row for the y ODE
MAT( [(N-1)*(J-1)+1 : (N-1)*(J-1)+J] , [(N-1)*(J-1)+1 : (N-1)*(J-1)+J] ) = lastrow_yPART;
for M1 = 1:N-1
    % install the other blocks (upside-down triangle symbol in the notes)
    lastrow_OTHERS = zeros(J,J-1); 
    lastrow_OTHERS([1:J-1],:) = (2*p4/(epxi*L)) * ((2*(-1)^(N+M1))/(1+x(M1+1))) * ERmat;
    lastrow_OTHERS(J,J-1) = - pi*p2 * sqrt(1-x(M1+1)^2) / (2*(xi^2)*dR*N);
    MAT( [(N-1)*(J-1)+1 : (N-1)*(J-1)+J] , [(M1-1)*(J-1)+1 : M1*(J-1)] ) = lastrow_OTHERS;
end
clear v0 v1 v2 M1 M2 ERvec ERmat DNorig DN DN2 yPART mainPART lastrow_yPART lastrow_OTHERS
%figure; spy(MAT)   % view sparse matrix structure


% (3) eigen values and vectors
[evecsALL,DD] = eig(MAT);
eig_res1 = max(max(abs(MAT*evecsALL - evecsALL*DD)));
%disp(['residual MAT.v - v.diag(eigenvalues) = ' num2str(eig_res1) ])
eig_cond1 = cond(evecsALL);
%disp(['condition number of eigenvector matrix ' num2str(eig_cond1) ])
evalsALL = diag(DD);   % extract eigenvalues from diagonal of DD
clear DD

% find eigenvalue with minimum real part and the associated eigenvector (evec)
[dum1,dum2] = min(abs(real(evalsALL)));
ssq = evalsALL(dum2);    % ssq (= s-squared)
evec = evecsALL(:,dum2); % associated eigenvector
eig_res2 = max(abs(MAT*evec - ssq*evec));
%disp(['max(abs(MAT.v - eig v)): ' num2str(eig_res2) ])
clear MAT evecsALL dum1 dum2

% enhancement factor
kr = sqrt(real(ssq));
f = 1 + (kr./kz).^2;                % enhancement factor
%disp(['enhancement factor f = ' num2str(f) ])

% extract-&-build z(R) and y(R) [note: use TRANSPOSE to avoid complex conjugation!]
z = transpose( [ reshape( evec([1:(N-1)*(J-1)]) , J-1 , N-1 ) ; zeros(1,N-1) ] );
z = [ zeros(1,J) ; z ; zeros(1,J) ];
yvec = transpose( evec([(N-1)*(J-1)+1 : end]) );
y = yvec;
H = z + repmat(y,N+1,1);

% sign correction & normalisation (using grain-interior value of y and H, which generally are complex)
z = z * sign(real(yvec(1)));
y = y * sign(real(yvec(1)));
H = H * sign(real(yvec(1)));

% normalisation to mid-grain interior value
Nmid = round(N/2) + 1;
y = y / H(Nmid,1);
z = z / H(Nmid,1);
H = H / H(Nmid,1);

clear Nmid yvec

% (4) save results
save results

